@@ -692,3 +692,65 @@ function check_extra_labels() {
     rm -f /tmp/exxx
   done
 }
+
+
+#Function: insert_tolerations
+#When use_spot_vm set to true, 
+#a tolerations should be set in helm chart template file for deploy/sts/ds/job/cronjob/pod.
+function insert_tolerations() {
+  local chart_folder=$1
+  #echo "[DEBUG] Updating helm chart [$chart_folder] templates to insert tolerations."
+  yamls=$(find -L $chart_folder/*/templates -name '*.yaml')
+  kind_array=("Deployment StatefulSet DaemonSet Job CronJob Pod")
+  
+  cd $CHART_FOLDER/temp
+  for template in ${yamls}
+  do
+    # Check the kind of each yaml file, only handle deploy/sts/ds/job/cronjob/pod kinds
+    kind=$(grep "^kind: " $template|awk -F ': ' {'print $2'})
+    if [[ "${kind_array[*]}" =~ "${kind}" ]]; then
+      #echo "[Debug] Setting tolerations for $kind : $template"
+      
+      # Ensure to insert "tolerations:" keyword into the template
+      value_tolerations_count=$(grep ".Values.[^\.]*[.]*tolerations" $template|wc -l)
+      if [[ $value_tolerations_count -eq 0 ]]; then
+        # No tolerations field defined in Values.yaml. 
+        # In this case, we assume there could be some existing tolerations defined directly within the template.
+        if [[ $kind == "Pod" ]]; then
+          # Specifically handling for cms-migrate Pod
+          sed -i '/^spec:/a\  tolerations:' $template
+        elif [[ $kind == "CronJob" ]]; then
+          grep -q "^          tolerations:$" $template || sed -i '/^        spec:/a\          tolerations:' $template
+        else
+          grep -q "^      tolerations:$" $template || sed -i '/^    spec:/a\      tolerations:' $template
+        fi
+      else
+        # There is tolerations field defined in Values.yaml.
+        # However, based on our CMS current chart design, tolerations field defined in Values.yaml never being truly used.
+        # So in this case, we can assume there is no any existing tolerations defined directly within the template and simply inset the keywork.
+        if [[ $kind == "CronJob" ]]; then
+          sed -i '/^        spec:/a\          tolerations:' $template
+        else
+          sed -i '/^    spec:/a\      tolerations:' $template
+        fi
+      fi
+      
+      # The INSERT_SPOT_TOLERATIONS ENV was deined from scripts load_cms_charts.sh
+      if $INSERT_SPOT_TOLERATIONS ; then
+        # Insert spot toleration into the template under "tolerations:" keyword.
+        # The file '$CHART_FOLDER/temp/spot_toleration.txt' was created in scripts load_cms_charts.sh.
+        #echo "[Debug] Adding spot_toleration for [$template]"
+        if [[ $kind == "Pod" ]]; then
+          # Specifically handling for cms-migrate Pod
+          sed -i '/^  tolerations:$/r spot_toleration.txt' $template
+        elif [[ $kind == "CronJob" ]]; then
+          sed -i '/^          tolerations:$/r spot_toleration_cronjob.txt' $template
+        else
+          sed -i '/^      tolerations:$/r spot_toleration.txt' $template
+        fi
+      fi
+    fi
+  done
+  cd - > /dev/null
+
+}
